﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Runtime.Serialization;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using MyTodos.Resources;

using Microsoft.WindowsAzure.MobileServices;
using Microsoft.Live;

namespace MyTodos
{
    public class TodoItem
    {
        public int Id { get; set; }

        [DataMember(Name = "text")]
        public string Text { get; set; }

        [DataMember(Name = "complete")]
        public bool Complete { get; set; }

        [DataMember(Name = "channel")]
        public string Channel { get; set; }
    }

    public partial class MainPage : PhoneApplicationPage
    {
        // MobileServiceCollectionView implements ICollectionView (useful for databinding to lists) and 
        // is integrated with your Mobile Service to make it easy to bind your data to the ListView
        private MobileServiceCollectionView<TodoItem> items;

        private IMobileServiceTable<TodoItem> todoTable = App.MobileService.GetTable<TodoItem>();

        // Constructor
        public MainPage()
        {
            InitializeComponent();
            //this.Loaded += MainPage_Loaded;
        }

        private async void InsertTodoItem(TodoItem todoItem)
        {
            // This code inserts a new TodoItem into the database. When the operation completes
            // and Mobile Services has assigned an Id, the item is added to the CollectionView
            await todoTable.InsertAsync(todoItem);
            items.Add(todoItem);
        }

        private void RefreshTodoItems()
        {
            // This code refreshes the entries in the list view be querying the TodoItems table.
            // The query excludes completed TodoItems
            items = todoTable
                .Where(todoItem => todoItem.Complete == false)
                .ToCollectionView();
            ListItems.ItemsSource = items;
        }

        private async void UpdateCheckedTodoItem(TodoItem item)
        {
            // This code takes a freshly completed TodoItem and updates the database. When the MobileService 
            // responds, the item is removed from the list 
            await todoTable.UpdateAsync(item);
            items.Remove(item);
        }

        private void ButtonRefresh_Click(object sender, RoutedEventArgs e)
        {
            RefreshTodoItems();
        }

        private void ButtonSave_Click(object sender, RoutedEventArgs e)
        {
            var todoItem = new TodoItem
            {
                Text = TodoInput.Text,
                Channel = App.CurrentChannel.ChannelUri.ToString()
            };
            InsertTodoItem(todoItem);
        }

        private void CheckBoxComplete_Checked(object sender, RoutedEventArgs e)
        {
            CheckBox cb = (CheckBox)sender;
            TodoItem item = cb.DataContext as TodoItem;
            item.Complete = true;
            UpdateCheckedTodoItem(item);
        }

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            //RefreshTodoItems();
        }

        private MobileServiceUser user;

        //private async System.Threading.Tasks.Task Authenticate()
        //{
        //    if (user == null)
        //    {
        //        string message = "You must log in. Login Required";
        //        try
        //        {
        //            user = await App.MobileService.LoginAsync(MobileServiceAuthenticationProvider.MicrosoftAccount);
        //            message = string.Format("You are now logged in - {0}", user.UserId);

        //        }
        //        catch (InvalidOperationException ex)
        //        {
        //            message = ex.Message;
        //        }

        //        MessageBox.Show(message);
        //    }
        //}

        private async void BtnLogin_Click(object sender, RoutedEventArgs e)
        {
            await Authenticate();

            if (null != this.user)
            {
                RefreshTodoItems();
            }
        }

        private async System.Threading.Tasks.Task Authenticate()
        {
            if (user == null)
            {
                string message = "You must log in. Login Required";
                try
                {
                    string clientId = "00000000480EEE41";
                    LiveAuthClient liveAuthClient = new LiveAuthClient(clientId);
                    LiveLoginResult result = await liveAuthClient.LoginAsync(new[] { "wl.basic", "wl.photos" });
                    if (result.Status == LiveConnectSessionStatus.Connected)
                    {
                        LiveConnectClient liveConnectClient = new LiveConnectClient(result.Session);
                        var me = await liveConnectClient.GetAsync("me");
                        user = await App.MobileService.LoginAsync(result.Session.AuthenticationToken); //(MobileServiceAuthenticationProvider.MicrosoftAccount);
                        
                        this.TitleTextBlock.Text = this.TitleTextBlock.Text +
                            " - welcome, "
                            + me.Result["first_name"].ToString() + "." + me.Result["last_name"].ToString();

                        message = string.Format("You are now logged in - {0}", user.UserId);
                    }
                    else
                    {
                        message = string.Format("Connect to microsoft account failed. Status: {0}", result.Status);
                    }

                }
                catch (LiveAuthException authEx)
                {
                    message = string.Format("{0}:{1}", authEx.GetType().Name, HttpUtility.UrlDecode(authEx.Message));
                }
                catch (LiveConnectException connEx)
                {
                    message = string.Format("{0}:{1}", connEx.GetType().Name, HttpUtility.UrlDecode(connEx.Message));
                }
                catch (InvalidOperationException ex)
                {
                    message = ex.Message;
                }

                MessageBox.Show(message);
            }
        }
    }
}